@@ -6,17 +6,17 @@
 
 #pragma once
 
-#include <QObject>
-
 #include "GpuDevice.h"
 #include "NvidiaSmiProcess.h"
 
+struct udev_device;
+
 class LinuxNvidiaGpu : public GpuDevice
 {
     Q_OBJECT
 
 public:
-    LinuxNvidiaGpu(int index, const QString& id, const QString& name);
+    LinuxNvidiaGpu(const QString& id, const QString &name, udev_device *device);
     ~LinuxNvidiaGpu() override;
 
     void initialize() override;
@@ -24,7 +24,8 @@ public:
 private:
     void onDataReceived(const NvidiaSmiProcess::GpuData &data);
 
-    int m_index = 0;
+    int m_index = -1;
+    udev_device *m_device;
 
     static NvidiaSmiProcess *s_smiProcess;
 };
